@@ -354,7 +354,7 @@ mod avx512 {
 
         #[inline(always)]
         unsafe fn _mm_loadu(data: *const f32) -> __m512i {
-            _f32_as_m512i_to_i32ord(_mm512_loadu_si512(data as *const i32))
+            _f32_as_m512i_to_i32ord(_mm512_loadu_si512(data as *const _))
         }
 
         #[inline(always)]
