@@ -80,11 +80,14 @@ pub enum XlsbError {
     Password,
     /// Worksheet not found
     WorksheetNotFound(String),
+    /// XML Encoding error
+    Encoding(quick_xml::encoding::EncodingError),
 }
 
 from_err!(std::io::Error, XlsbError, Io);
 from_err!(zip::result::ZipError, XlsbError, Zip);
 from_err!(quick_xml::Error, XlsbError, Xml);
+from_err!(quick_xml::encoding::EncodingError, XlsbError, Encoding);
 
 impl std::fmt::Display for XlsbError {
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
@@ -114,6 +117,7 @@ impl std::fmt::Display for XlsbError {
             }
             XlsbError::Password => write!(f, "Workbook is password protected"),
             XlsbError::WorksheetNotFound(name) => write!(f, "Worksheet '{name}' not found"),
+            XlsbError::Encoding(e) => write!(f, "XML encoding error: {e}"),
         }
     }
 }
@@ -159,10 +163,11 @@ impl<RS: Read + Seek> Xlsb<RS> {
         match self.zip.by_name("xl/_rels/workbook.bin.rels") {
             Ok(f) => {
                 let mut xml = XmlReader::from_reader(BufReader::new(f));
-                xml.check_end_names(false)
-                    .trim_text(false)
-                    .check_comments(false)
-                    .expand_empty_elements(true);
+                let config = xml.config_mut();
+                config.check_end_names = false;
+                config.trim_text(false);
+                config.check_comments = false;
+                config.expand_empty_elements = true;
                 let mut buf: Vec<u8> = Vec::with_capacity(64);
 
                 loop {
@@ -182,7 +187,12 @@ impl<RS: Read + Seek> Xlsb<RS> {
                                         key: QName(b"Target"),
                                         value: v,
                                     } => {
-                                        target = Some(xml.decoder().decode(&v)?.into_owned());
+                                        target = Some(
+                                            xml.decoder()
+                                                .decode(&v)
+                                                .map_err(XlsbError::Encoding)?
+                                                .into_owned(),
+                                        );
                                     }
                                     _ => (),
                                 }
@@ -398,7 +408,7 @@ impl<RS: Read + Seek> Xlsb<RS> {
     pub fn worksheet_cells_reader<'a>(
         &'a mut self,
         name: &str,
-    ) -> Result<XlsbCellsReader<'a>, XlsbError> {
+    ) -> Result<XlsbCellsReader<'a, RS>, XlsbError> {
         let path = match self.sheets.iter().find(|&(n, _)| n == name) {
             Some((_, path)) => path.clone(),
             None => return Err(XlsbError::WorksheetNotFound(name.into())),
@@ -598,16 +608,19 @@ impl<RS: Read + Seek> ReaderRef<RS> for Xlsb<RS> {
     }
 }
 
-pub(crate) struct RecordIter<'a> {
+pub(crate) struct RecordIter<'a, RS>
+where
+    RS: Read + Seek,
+{
     b: [u8; 1],
-    r: BufReader<ZipFile<'a>>,
+    r: BufReader<ZipFile<'a, RS>>,
 }
 
-impl<'a> RecordIter<'a> {
-    fn from_zip<RS: Read + Seek>(
-        zip: &'a mut ZipArchive<RS>,
-        path: &str,
-    ) -> Result<RecordIter<'a>, XlsbError> {
+impl<'a, RS> RecordIter<'a, RS>
+where
+    RS: Read + Seek,
+{
+    fn from_zip(zip: &'a mut ZipArchive<RS>, path: &str) -> Result<RecordIter<'a, RS>, XlsbError> {
         match zip.by_name(path) {
             Ok(f) => Ok(RecordIter {
                 r: BufReader::new(f),
