@@ -11,15 +11,18 @@ use crate::{
     ansi::{ANSIBuf, ANSIFmt},
     colors::Colors,
     config::{
-        spanned::{Offset, SpannedConfig},
-        Formatting,
+        spanned::SpannedConfig, AlignmentHorizontal, AlignmentVertical, Formatting, Indent, Offset,
+        Position, Sides,
     },
-    config::{AlignmentHorizontal, AlignmentVertical, Entity, Indent, Position, Sides},
     dimension::Dimension,
     records::{ExactRecords, PeekableRecords, Records},
     util::string::get_line_width,
 };
 
+// TODO: Do we actually need PeekableRecords?
+//       Maybe Cloned iterator will be faster?
+//       Even better &referenced Records
+
 /// Grid provides a set of methods for building a text-based table.
 #[derive(Debug, Clone)]
 pub struct PeekableGrid<R, G, D, C> {
@@ -43,7 +46,7 @@ impl<R, G, D, C> PeekableGrid<R, G, D, C> {
 
 impl<R, G, D, C> PeekableGrid<R, G, D, C> {
     /// Builds a table.
-    pub fn build<F>(self, mut f: F) -> fmt::Result
+    pub fn build<F>(&self, mut f: F) -> fmt::Result
     where
         R: Records + PeekableRecords + ExactRecords,
         D: Dimension,
@@ -64,21 +67,17 @@ impl<R, G, D, C> PeekableGrid<R, G, D, C> {
 
         print_grid(&mut f, ctx)
     }
+}
 
-    /// Builds a table into string.
-    ///
-    /// Notice that it consumes self.
-    #[allow(clippy::inherent_to_string)]
-    pub fn to_string(self) -> String
-    where
-        R: Records + PeekableRecords + ExactRecords,
-        D: Dimension,
-        G: Borrow<SpannedConfig>,
-        C: Colors,
-    {
-        let mut buf = String::new();
-        self.build(&mut buf).expect("It's guaranteed to never happen otherwise it's considered an stdlib error or impl error");
-        buf
+impl<R, G, D, C> fmt::Display for PeekableGrid<R, G, D, C>
+where
+    R: Records + PeekableRecords + ExactRecords,
+    D: Dimension,
+    C: Colors,
+    G: Borrow<SpannedConfig>,
+{
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        self.build(f)
     }
 }
 
@@ -103,11 +102,10 @@ where
     }
 
     let is_basic = !ctx.cfg.has_border_colors()
+        && !ctx.cfg.has_padding_color()
         && !ctx.cfg.has_justification()
-        && ctx.cfg.get_justification_color(Entity::Global).is_none()
         && !ctx.cfg.has_offset_chars()
         && !has_margin(ctx.cfg)
-        && !has_padding_color(ctx.cfg)
         && ctx.colors.is_empty();
 
     if is_basic {
@@ -122,14 +120,6 @@ fn has_margin(cfg: &SpannedConfig) -> bool {
     margin.left.size > 0 || margin.right.size > 0 || margin.top.size > 0 || margin.bottom.size > 0
 }
 
-fn has_padding_color(cfg: &SpannedConfig) -> bool {
-    let pad = cfg.get_padding_color(Entity::Global);
-    let has_pad =
-        pad.left.is_some() || pad.right.is_some() || pad.top.is_some() || pad.bottom.is_some();
-
-    has_pad || cfg.has_padding_color()
-}
-
 mod grid_basic {
     use super::*;
 
@@ -218,12 +208,12 @@ mod grid_basic {
         D: Dimension,
     {
         for col in 0..shape.count_columns {
-            let pos = (row, col);
+            let pos = (row, col).into();
             print_vertical_char(f, ctx.cfg, pos, shape.count_columns)?;
             print_cell_line(f, ctx, height, pos, line)?;
         }
 
-        let pos = (row, shape.count_columns);
+        let pos = (row, shape.count_columns).into();
         print_vertical_char(f, ctx.cfg, pos, shape.count_columns)?;
 
         Ok(())
@@ -240,14 +230,14 @@ mod grid_basic {
         F: Write,
         D: Dimension,
     {
-        print_vertical_intersection(f, cfg, (row, 0), shape)?;
+        print_vertical_intersection(f, cfg, (row, 0).into(), shape)?;
 
         for col in 0..shape.count_columns {
             let width = dimension.get_width(col);
 
             // general case
             if width > 0 {
-                let pos = (row, col);
+                let pos = (row, col).into();
                 let main = cfg.get_horizontal(pos, shape.count_rows);
                 match main {
                     Some(c) => repeat_char(f, c, width)?,
@@ -255,7 +245,7 @@ mod grid_basic {
                 }
             }
 
-            let pos = (row, col + 1);
+            let pos = (row, col + 1).into();
             print_vertical_intersection(f, cfg, pos, shape)?;
         }
 
@@ -282,7 +272,7 @@ mod grid_basic {
         //
         // todo: Yes... this check very likely degrages performance a bit,
         //       Surely we need to rethink it.
-        if !cfg.has_vertical(pos.1, shape.count_columns) {
+        if !cfg.has_vertical(pos.col, shape.count_columns) {
             return Ok(());
         }
 
@@ -322,16 +312,14 @@ mod grid_basic {
         R: Records + PeekableRecords + ExactRecords,
         D: Dimension,
     {
-        let entity = Entity::from(pos);
+        let width = ctx.dims.get_width(pos.col);
 
-        let width = ctx.dims.get_width(pos.1);
-
-        let pad = ctx.cfg.get_padding(entity);
-        let valignment = *ctx.cfg.get_alignment_vertical(entity);
+        let pad = ctx.cfg.get_padding(pos);
+        let valignment = *ctx.cfg.get_alignment_vertical(pos);
         let text_cfg = TextCfg {
-            alignment: *ctx.cfg.get_alignment_horizontal(entity),
-            formatting: *ctx.cfg.get_formatting(entity),
-            justification: ctx.cfg.get_justification(Entity::Global),
+            alignment: *ctx.cfg.get_alignment_horizontal(pos),
+            formatting: ctx.cfg.get_formatting(pos),
+            justification: ctx.cfg.get_justification(pos),
         };
 
         let mut cell_height = ctx.records.count_lines(pos);
@@ -345,7 +333,7 @@ mod grid_basic {
             cell_height = height;
         }
 
-        let indent = top_indent(&pad, valignment, cell_height, height);
+        let indent = top_indent(pad, valignment, cell_height, height);
         if indent > line {
             return repeat_char(f, pad.top.fill, width);
         }
@@ -415,9 +403,8 @@ mod grid_basic {
         let indent = calculate_indent(cfg.alignment, cell_width, available);
         print_text_padded(f, &line, cfg.justification, indent)?;
 
-        // todo: remove me?
         let rest_width = cell_width - line_width;
-        repeat_char(f, ' ', rest_width)?;
+        repeat_char(f, cfg.justification, rest_width)?;
 
         Ok(())
     }
@@ -602,12 +589,13 @@ mod grid_not_spanned {
                 print_margin_left(f, ctx.cfg, table_line, total_height)?;
 
                 for col in 0..shape.count_columns {
-                    print_vertical_char(f, ctx.cfg, (row, col), i, height, shape.count_columns)?;
-                    print_cell_line(f, &ctx, height, (row, col), i)?;
+                    let pos = (row, col).into();
+                    print_vertical_char(f, ctx.cfg, pos, i, height, shape.count_columns)?;
+                    print_cell_line(f, &ctx, height, pos, i)?;
 
                     let is_last_column = col + 1 == shape.count_columns;
                     if is_last_column {
-                        let pos = (row, col + 1);
+                        let pos = (row, col + 1).into();
                         print_vertical_char(f, ctx.cfg, pos, i, height, shape.count_columns)?;
                     }
                 }
@@ -651,14 +639,14 @@ mod grid_not_spanned {
         D: Dimension,
     {
         let mut used_color = None;
-        print_vertical_intersection(f, cfg, (row, 0), shape, &mut used_color)?;
+        print_vertical_intersection(f, cfg, (row, 0).into(), shape, &mut used_color)?;
 
         for col in 0..shape.count_columns {
             let width = dimension.get_width(col);
 
             // general case
             if width > 0 {
-                let pos = (row, col);
+                let pos = (row, col).into();
                 let main = cfg.get_horizontal(pos, shape.count_rows);
                 match main {
                     Some(c) => {
@@ -670,7 +658,7 @@ mod grid_not_spanned {
                 }
             }
 
-            let pos = (row, col + 1);
+            let pos = (row, col + 1).into();
             print_vertical_intersection(f, cfg, pos, shape, &mut used_color)?;
         }
 
@@ -702,7 +690,7 @@ mod grid_not_spanned {
         //
         // todo: Yes... this check very likely degrages performance a bit,
         //       Surely we need to rethink it.
-        if !cfg.has_vertical(pos.1, shape.count_columns) {
+        if !cfg.has_vertical(pos.col, shape.count_columns) {
             return Ok(());
         }
 
@@ -833,24 +821,22 @@ mod grid_not_spanned {
         C: Colors,
         D: Dimension,
     {
-        let entity = pos.into();
+        let width = ctx.dims.get_width(pos.col);
 
-        let width = ctx.dims.get_width(pos.1);
-
-        let formatting = ctx.cfg.get_formatting(entity);
+        let formatting = ctx.cfg.get_formatting(pos);
         let text_cfg = TextCfg {
-            alignment: *ctx.cfg.get_alignment_horizontal(entity),
+            alignment: *ctx.cfg.get_alignment_horizontal(pos),
             color: ctx.colors.get_color(pos),
             justification: Colored::new(
-                ctx.cfg.get_justification(entity),
-                ctx.cfg.get_justification_color(entity),
+                ctx.cfg.get_justification(pos),
+                ctx.cfg.get_justification_color(pos),
             ),
-            formatting: *formatting,
+            formatting,
         };
 
-        let pad = ctx.cfg.get_padding(entity);
-        let pad_color = ctx.cfg.get_padding_color(entity);
-        let valignment = *ctx.cfg.get_alignment_vertical(entity);
+        let pad = ctx.cfg.get_padding(pos);
+        let pad_color = ctx.cfg.get_padding_color(pos);
+        let valignment = *ctx.cfg.get_alignment_vertical(pos);
 
         let mut cell_height = ctx.records.count_lines(pos);
         if formatting.vertical_trim {
@@ -863,7 +849,7 @@ mod grid_not_spanned {
             cell_height = height;
         }
 
-        let indent = top_indent(&pad, valignment, cell_height, height);
+        let indent = top_indent(pad, valignment, cell_height, height);
         if indent > line {
             return print_indent(f, pad.top.fill, width, pad_color.top.as_ref());
         }
@@ -919,7 +905,7 @@ mod grid_not_spanned {
         if cfg.formatting.allow_lines_alignment {
             let indent = calculate_indent(cfg.alignment, line_width, available);
             let text = Colored::new(line.as_ref(), cfg.color);
-            return print_text_padded(f, text, cfg.justification, indent);
+            return print_text_padded(f, &text, &cfg.justification, indent);
         }
 
         let cell_width = if cfg.formatting.horizontal_trim {
@@ -934,19 +920,18 @@ mod grid_not_spanned {
 
         let indent = calculate_indent(cfg.alignment, cell_width, available);
         let text = Colored::new(line.as_ref(), cfg.color);
-        print_text_padded(f, text, cfg.justification, indent)?;
+        print_text_padded(f, &text, &cfg.justification, indent)?;
 
-        // todo: remove me?
         let rest_width = cell_width - line_width;
-        repeat_char(f, ' ', rest_width)?;
+        print_indent2(f, &cfg.justification, rest_width)?;
 
         Ok(())
     }
 
     fn print_text_padded<F, C, C1>(
         f: &mut F,
-        text: Colored<&str, C>,
-        justification: Colored<char, C1>,
+        text: &Colored<&str, C>,
+        justification: &Colored<char, C1>,
         indent: HIndent,
     ) -> fmt::Result
     where
@@ -954,19 +939,19 @@ mod grid_not_spanned {
         C: ANSIFmt,
         C1: ANSIFmt,
     {
-        print_indent2(f, &justification, indent.left)?;
+        print_indent2(f, justification, indent.left)?;
         print_text2(f, text)?;
-        print_indent2(f, &justification, indent.right)?;
+        print_indent2(f, justification, indent.right)?;
 
         Ok(())
     }
 
-    fn print_text2<F, C>(f: &mut F, text: Colored<&str, C>) -> fmt::Result
+    fn print_text2<F, C>(f: &mut F, text: &Colored<&str, C>) -> fmt::Result
     where
         F: Write,
         C: ANSIFmt,
     {
-        match text.color {
+        match &text.color {
             Some(color) => {
                 color.fmt_ansi_prefix(f)?;
                 f.write_str(text.data)?;
@@ -1071,7 +1056,7 @@ mod grid_not_spanned {
         let indent = cfg.get_margin().top;
         let offset = cfg.get_margin_offset().top;
         let color = cfg.get_margin_color();
-        let color = color.top.as_ref();
+        let color = color.top;
         print_indent_lines(f, indent, offset, color, width)
     }
 
@@ -1082,7 +1067,7 @@ mod grid_not_spanned {
         let indent = cfg.get_margin().bottom;
         let offset = cfg.get_margin_offset().bottom;
         let color = cfg.get_margin_color();
-        let color = color.bottom.as_ref();
+        let color = color.bottom;
         print_indent_lines(f, indent, offset, color, width)
     }
 
@@ -1098,7 +1083,7 @@ mod grid_not_spanned {
         let indent = cfg.get_margin().left;
         let offset = cfg.get_margin_offset().left;
         let color = cfg.get_margin_color();
-        let color = color.left.as_ref();
+        let color = color.left;
         print_margin_vertical(f, indent, offset, color, line, height)
     }
 
@@ -1114,7 +1099,7 @@ mod grid_not_spanned {
         let indent = cfg.get_margin().right;
         let offset = cfg.get_margin_offset().right;
         let color = cfg.get_margin_color();
-        let color = color.right.as_ref();
+        let color = color.right;
         print_margin_vertical(f, indent, offset, color, line, height)
     }
 
@@ -1134,7 +1119,7 @@ mod grid_not_spanned {
         }
 
         match offset {
-            Offset::Begin(offset) => {
+            Offset::Start(offset) => {
                 let offset = cmp::min(offset, height);
                 if line >= offset {
                     print_indent(f, indent.fill, indent.size, color)?;
@@ -1172,7 +1157,7 @@ mod grid_not_spanned {
         }
 
         let (start_offset, end_offset) = match offset {
-            Offset::Begin(start) => (start, 0),
+            Offset::Start(start) => (start, 0),
             Offset::End(end) => (0, end),
         };
 
@@ -1340,7 +1325,7 @@ mod grid_spanned {
                 print_margin_left(f, ctx.cfg, table_line, total_height)?;
 
                 for col in 0..shape.count_columns {
-                    let pos = (row, col);
+                    let pos = (row, col).into();
 
                     if ctx.cfg.is_cell_covered_by_both_spans(pos) {
                         continue;
@@ -1349,7 +1334,7 @@ mod grid_spanned {
                     if ctx.cfg.is_cell_covered_by_column_span(pos) {
                         let is_last_column = col + 1 == shape.count_columns;
                         if is_last_column {
-                            let pos = (row, col + 1);
+                            let pos = (row, col + 1).into();
                             let count_columns = shape.count_columns;
                             print_vertical_char(f, ctx.cfg, pos, i, count_lines, count_columns)?;
                         }
@@ -1374,7 +1359,7 @@ mod grid_spanned {
                             .sum::<usize>();
 
                         let line = i + skip_lines;
-                        let pos = (original_row, col);
+                        let pos = (original_row, col).into();
 
                         let width = get_cell_width(ctx.cfg, ctx.dims, pos, shape.count_columns);
                         let height = get_cell_height(ctx.cfg, ctx.dims, pos, shape.count_rows);
@@ -1388,7 +1373,7 @@ mod grid_spanned {
 
                     let is_last_column = col + 1 == shape.count_columns;
                     if is_last_column {
-                        let pos = (row, col + 1);
+                        let pos = (row, col + 1).into();
                         print_vertical_char(f, ctx.cfg, pos, i, count_lines, shape.count_columns)?;
                     }
                 }
@@ -1434,11 +1419,11 @@ mod grid_spanned {
     {
         let mut used_color = None;
 
-        let pos = (row, 0);
+        let pos = (row, 0).into();
         print_vertical_intersection(f, ctx.cfg, pos, shape, &mut used_color)?;
 
         for col in 0..shape.count_columns {
-            let pos = (row, col);
+            let pos = (row, col).into();
             if ctx.cfg.is_cell_covered_by_both_spans(pos) {
                 continue;
             }
@@ -1447,6 +1432,8 @@ mod grid_spanned {
                 // means it's part of other a spanned cell
                 // so. we just need to use line from other cell.
 
+                prepare_coloring(f, None, &mut used_color)?;
+
                 let original_row = closest_visible_row(ctx.cfg, pos).unwrap();
 
                 // considering that the content will be printed instead horizontal lines so we can skip some lines.
@@ -1461,7 +1448,7 @@ mod grid_spanned {
                         .sum::<usize>();
                 }
 
-                let pos = (original_row, col);
+                let pos = (original_row, col).into();
                 let height = get_cell_height(ctx.cfg, ctx.dims, pos, shape.count_rows);
                 let width = get_cell_width(ctx.cfg, ctx.dims, pos, shape.count_columns);
                 let line = skip_lines;
@@ -1474,7 +1461,7 @@ mod grid_spanned {
                     col += span - 1;
                 }
 
-                let pos = (row, col + 1);
+                let pos = (row, col + 1).into();
                 print_vertical_intersection(f, ctx.cfg, pos, shape, &mut used_color)?;
 
                 continue;
@@ -1494,7 +1481,7 @@ mod grid_spanned {
                 }
             }
 
-            let pos = (row, col + 1);
+            let pos = (row, col + 1).into();
             print_vertical_intersection(f, ctx.cfg, pos, shape, &mut used_color)?;
         }
 
@@ -1562,7 +1549,7 @@ mod grid_spanned {
         //
         // todo: Yes... this check very likely degrages performance a bit,
         //       Surely we need to rethink it.
-        if !cfg.has_vertical(pos.1, shape.count_columns) {
+        if !cfg.has_vertical(pos.col, shape.count_columns) {
             return Ok(());
         }
 
@@ -1617,14 +1604,14 @@ mod grid_spanned {
         D: Dimension,
     {
         let mut used_color = None;
-        print_vertical_intersection(f, cfg, (row, 0), shape, &mut used_color)?;
+        print_vertical_intersection(f, cfg, (row, 0).into(), shape, &mut used_color)?;
 
         for col in 0..shape.count_columns {
             let width = dimension.get_width(col);
 
             // general case
             if width > 0 {
-                let pos = (row, col);
+                let pos = (row, col).into();
                 let main = cfg.get_horizontal(pos, shape.count_rows);
                 match main {
                     Some(c) => {
@@ -1636,7 +1623,7 @@ mod grid_spanned {
                 }
             }
 
-            let pos = (row, col + 1);
+            let pos = (row, col + 1).into();
             print_vertical_intersection(f, cfg, pos, shape, &mut used_color)?;
         }
 
@@ -1699,10 +1686,8 @@ mod grid_spanned {
         R: Records + PeekableRecords + ExactRecords,
         C: Colors,
     {
-        let entity = pos.into();
-
         let mut cell_height = ctx.records.count_lines(pos);
-        let formatting = ctx.cfg.get_formatting(entity);
+        let formatting = ctx.cfg.get_formatting(pos);
         if formatting.vertical_trim {
             cell_height -= count_empty_lines_at_start(ctx.records, pos)
                 + count_empty_lines_at_end(ctx.records, pos);
@@ -1713,10 +1698,10 @@ mod grid_spanned {
             cell_height = height;
         }
 
-        let pad = ctx.cfg.get_padding(entity);
-        let pad_color = ctx.cfg.get_padding_color(entity);
-        let alignment = ctx.cfg.get_alignment_vertical(entity);
-        let indent = top_indent(&pad, *alignment, cell_height, height);
+        let pad = ctx.cfg.get_padding(pos);
+        let pad_color = ctx.cfg.get_padding_color(pos);
+        let alignment = ctx.cfg.get_alignment_vertical(pos);
+        let indent = top_indent(pad, *alignment, cell_height, height);
         if indent > line {
             return print_indent(f, pad.top.fill, width, pad_color.top.as_ref());
         }
@@ -1742,13 +1727,13 @@ mod grid_spanned {
         let width = width - pad.left.size - pad.right.size;
 
         let line_cfg = TextCfg {
-            alignment: *ctx.cfg.get_alignment_horizontal(entity),
-            formatting: *formatting,
+            alignment: *ctx.cfg.get_alignment_horizontal(pos),
             color: ctx.colors.get_color(pos),
             justification: Colored::new(
-                ctx.cfg.get_justification(entity),
-                ctx.cfg.get_justification_color(entity),
+                ctx.cfg.get_justification(pos),
+                ctx.cfg.get_justification_color(pos),
             ),
+            formatting,
         };
 
         print_line(f, ctx.records, pos, index, width, line_cfg)?;
@@ -1785,7 +1770,7 @@ mod grid_spanned {
         if text_cfg.formatting.allow_lines_alignment {
             let indent = calculate_indent(text_cfg.alignment, line_width, available);
             let text = Colored::new(line.as_ref(), text_cfg.color);
-            return print_text_with_pad(f, text, text_cfg.justification, indent);
+            return print_text_with_pad(f, &text, &text_cfg.justification, indent);
         }
 
         let cell_width = if text_cfg.formatting.horizontal_trim {
@@ -1799,20 +1784,24 @@ mod grid_spanned {
         };
 
         let indent = calculate_indent(text_cfg.alignment, cell_width, available);
-        let text = Colored::new(line.as_ref(), text_cfg.color);
-        print_text_with_pad(f, text, text_cfg.justification, indent)?;
+        let text = Colored::new(line.as_ref(), text_cfg.color.as_ref());
+        print_text_with_pad(f, &text, &text_cfg.justification, indent)?;
 
-        // todo: remove me?
         let rest_width = cell_width - line_width;
-        repeat_char(f, ' ', rest_width)?;
+        print_indent(
+            f,
+            text_cfg.justification.data,
+            rest_width,
+            text_cfg.justification.color.as_ref(),
+        )?;
 
         Ok(())
     }
 
     fn print_text_with_pad<F, C, C1>(
         f: &mut F,
-        text: Colored<&str, C>,
-        space: Colored<char, C1>,
+        text: &Colored<&str, C>,
+        space: &Colored<char, C1>,
         indent: HIndent,
     ) -> fmt::Result
     where
@@ -1821,7 +1810,7 @@ mod grid_spanned {
         C1: ANSIFmt,
     {
         print_indent(f, space.data, indent.left, space.color.as_ref())?;
-        print_text(f, text.data, text.color)?;
+        print_text(f, text.data, text.color.as_ref())?;
         print_indent(f, space.data, indent.right, space.color.as_ref())?;
         Ok(())
     }
@@ -1936,7 +1925,7 @@ mod grid_spanned {
         let indent = cfg.get_margin().top;
         let offset = cfg.get_margin_offset().top;
         let color = cfg.get_margin_color();
-        let color = color.top.as_ref();
+        let color = color.top;
         print_indent_lines(f, &indent, &offset, color, width)
     }
 
@@ -1947,7 +1936,7 @@ mod grid_spanned {
         let indent = cfg.get_margin().bottom;
         let offset = cfg.get_margin_offset().bottom;
         let color = cfg.get_margin_color();
-        let color = color.bottom.as_ref();
+        let color = color.bottom;
         print_indent_lines(f, &indent, &offset, color, width)
     }
 
@@ -1963,7 +1952,7 @@ mod grid_spanned {
         let indent = cfg.get_margin().left;
         let offset = cfg.get_margin_offset().left;
         let color = cfg.get_margin_color();
-        let color = color.left.as_ref();
+        let color = color.left;
         print_margin_vertical(f, indent, offset, color, line, height)
     }
 
@@ -1979,7 +1968,7 @@ mod grid_spanned {
         let indent = cfg.get_margin().right;
         let offset = cfg.get_margin_offset().right;
         let color = cfg.get_margin_color();
-        let color = color.right.as_ref();
+        let color = color.right;
         print_margin_vertical(f, indent, offset, color, line, height)
     }
 
@@ -1999,7 +1988,7 @@ mod grid_spanned {
         }
 
         match offset {
-            Offset::Begin(offset) => {
+            Offset::Start(offset) => {
                 let offset = cmp::min(offset, height);
                 if line >= offset {
                     print_indent(f, indent.fill, indent.size, color)?;
@@ -2037,7 +2026,7 @@ mod grid_spanned {
         }
 
         let (start_offset, end_offset) = match offset {
-            Offset::Begin(start) => (*start, 0),
+            Offset::Start(start) => (*start, 0),
             Offset::End(end) => (0, *end),
         };
 
@@ -2091,11 +2080,11 @@ mod grid_spanned {
     {
         match cfg.get_column_span(pos) {
             Some(span) => {
-                let start = pos.1;
+                let start = pos.col;
                 let end = start + span;
                 range_width(dims, start, end) + count_verticals_range(cfg, start, end, max)
             }
-            None => dims.get_width(pos.1),
+            None => dims.get_width(pos.col),
         }
     }
 
@@ -2118,11 +2107,11 @@ mod grid_spanned {
     {
         match cfg.get_row_span(pos) {
             Some(span) => {
-                let start = pos.0;
-                let end = pos.0 + span;
+                let start = pos.row;
+                let end = pos.row + span;
                 range_height(dims, start, end) + count_horizontals_range(cfg, start, end, max)
             }
-            None => dims.get_height(pos.0),
+            None => dims.get_height(pos.row),
         }
     }
 
@@ -2142,14 +2131,14 @@ mod grid_spanned {
     fn closest_visible_row(cfg: &SpannedConfig, mut pos: Position) -> Option<usize> {
         loop {
             if cfg.is_cell_visible(pos) {
-                return Some(pos.0);
+                return Some(pos.row);
             }
 
-            if pos.0 == 0 {
+            if pos.row == 0 {
                 return None;
             }
 
-            pos.0 -= 1;
+            pos -= (1, 0);
         }
     }
 
