@@ -1,7 +1,7 @@
 use crate::{
     builder::Builder,
     traits::{Primitive, WriteAsOptional, WriteAsOptionalUnion, WriteAsPrimitive},
-    Cursor, UnionOffset, Void,
+    Cursor, UnionOffset, UnionVectorOffset, Void, WriteAsOptionalUnionVector,
 };
 
 impl<P: Primitive> WriteAsPrimitive<P> for Void {
@@ -25,3 +25,10 @@ impl<T: ?Sized> WriteAsOptionalUnion<T> for () {
         None
     }
 }
+
+impl<T: ?Sized> WriteAsOptionalUnionVector<T> for () {
+    #[inline]
+    fn prepare(&self, _builder: &mut Builder) -> Option<UnionVectorOffset<T>> {
+        None
+    }
+}
