@@ -1,11 +1,12 @@
+#![allow(unsafe_op_in_unsafe_fn)]
 use std::hint::unreachable_unchecked;
 
-use polars_error::{polars_bail, PolarsResult};
+use polars_error::{PolarsResult, polars_bail};
 use polars_utils::vec::PushUnchecked;
 
 use super::bitmask::BitMask;
-use super::utils::{count_zeros, fmt, BitChunk, BitChunks, BitChunksExactMut, BitmapIter};
-use super::{intersects_with_mut, Bitmap};
+use super::utils::{BitChunk, BitChunks, BitChunksExactMut, BitmapIter, count_zeros, fmt};
+use super::{Bitmap, intersects_with_mut};
 use crate::bitmap::utils::{get_bit_unchecked, merge_reversed, set_bit_in_byte};
 use crate::storage::SharedStorage;
 use crate::trusted_len::TrustedLen;
@@ -331,7 +332,7 @@ impl MutableBitmap {
             let required = (self.length + additional).saturating_add(7) / 8;
             // add remaining as full bytes
             self.buffer
-                .extend(std::iter::repeat(0b11111111u8).take(required - existing));
+                .extend(std::iter::repeat_n(0b11111111u8, required - existing));
             self.length += additional;
         }
     }
@@ -555,7 +556,7 @@ unsafe fn extend_aligned_trusted_iter_unchecked(
     let chunks = additional_bits / 64;
     let remainder = additional_bits % 64;
 
-    let additional = (additional_bits + 7) / 8;
+    let additional = additional_bits.div_ceil(8);
     assert_eq!(
         additional,
         // a hint of how the following calculation will be done
@@ -687,7 +688,7 @@ impl MutableBitmap {
     {
         let length = iterator.size_hint().1.unwrap();
 
-        let mut buffer = vec![0u8; (length + 7) / 8];
+        let mut buffer = vec![0u8; length.div_ceil(8)];
 
         let chunks = length / 8;
         let reminder = length % 8;
