@@ -30,6 +30,7 @@ pub(crate) type Extension = Option<(PlSmallStr, Option<PlSmallStr>)>;
 /// Use `to_logical_type` to desugar such type and return its corresponding logical type.
 #[derive(Debug, Clone, PartialEq, Eq, Hash, Default)]
 #[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
+#[cfg_attr(feature = "dsl-schema", derive(schemars::JsonSchema))]
 pub enum ArrowDataType {
     /// Null type
     #[default]
@@ -157,6 +158,10 @@ pub enum ArrowDataType {
     /// scale is the number of decimal places.
     /// The number 999.99 has a precision of 5 and scale of 2.
     Decimal(usize, usize),
+    /// Decimal backed by 32 bits
+    Decimal32(usize, usize),
+    /// Decimal backed by 64 bits
+    Decimal64(usize, usize),
     /// Decimal backed by 256 bits
     Decimal256(usize, usize),
     /// Extension type.
@@ -171,12 +176,13 @@ pub enum ArrowDataType {
     Unknown,
     /// A nested datatype that can represent slots of differing types.
     /// Third argument represents mode
-    #[cfg_attr(feature = "serde", serde(skip))]
+    #[cfg_attr(any(feature = "serde", feature = "dsl-schema"), serde(skip))]
     Union(Box<UnionType>),
 }
 
 #[derive(Debug, Clone, PartialEq, Eq, Hash)]
 #[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
+#[cfg_attr(feature = "dsl-schema", derive(schemars::JsonSchema))]
 pub struct ExtensionType {
     pub name: PlSmallStr,
     pub inner: ArrowDataType,
@@ -221,6 +227,7 @@ impl UnionMode {
 /// The time units defined in Arrow.
 #[derive(Debug, Clone, Copy, PartialEq, Eq, Hash)]
 #[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
+#[cfg_attr(feature = "dsl-schema", derive(schemars::JsonSchema))]
 pub enum TimeUnit {
     /// Time in seconds.
     Second,
@@ -235,6 +242,7 @@ pub enum TimeUnit {
 /// Interval units defined in Arrow
 #[derive(Debug, Clone, Copy, PartialEq, Eq, Hash)]
 #[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
+#[cfg_attr(feature = "dsl-schema", derive(schemars::JsonSchema))]
 pub enum IntervalUnit {
     /// The number of elapsed whole months.
     YearMonth,
@@ -273,6 +281,8 @@ impl ArrowDataType {
                 PhysicalType::Primitive(PrimitiveType::Int64)
             },
             Decimal(_, _) => PhysicalType::Primitive(PrimitiveType::Int128),
+            Decimal32(_, _) => PhysicalType::Primitive(PrimitiveType::Int32),
+            Decimal64(_, _) => PhysicalType::Primitive(PrimitiveType::Int64),
             Decimal256(_, _) => PhysicalType::Primitive(PrimitiveType::Int256),
             UInt8 => PhysicalType::Primitive(PrimitiveType::UInt8),
             UInt16 => PhysicalType::Primitive(PrimitiveType::UInt16),
@@ -405,6 +415,8 @@ impl ArrowDataType {
                 | D::Float32
                 | D::Float64
                 | D::Decimal(_, _)
+                | D::Decimal32(_, _)
+                | D::Decimal64(_, _)
                 | D::Decimal256(_, _)
         )
     }
@@ -451,6 +463,8 @@ impl ArrowDataType {
             | D::Utf8
             | D::LargeUtf8
             | D::Decimal(_, _)
+            | D::Decimal32(_, _)
+            | D::Decimal64(_, _)
             | D::Decimal256(_, _)
             | D::BinaryView
             | D::Utf8View
