@@ -364,15 +364,32 @@ unsafe fn to_dtype(schema: &ArrowSchema) -> PolarsResult<ArrowDataType> {
                             let bit_width = width_raw.parse::<usize>().map_err(|_| {
                                 polars_err!(ComputeError: "Decimal bit width is not a valid integer")
                             })?;
-                            if bit_width == 256 {
-                                return Ok(ArrowDataType::Decimal256(
+                            match bit_width {
+                                32 => return Ok(ArrowDataType::Decimal32(
                                     precision_raw.parse::<usize>().map_err(|_| {
                                         polars_err!(ComputeError: "Decimal precision is not a valid integer")
                                     })?,
                                     scale_raw.parse::<usize>().map_err(|_| {
                                         polars_err!(ComputeError: "Decimal scale is not a valid integer")
                                     })?,
-                                ));
+                                )),
+                                64 => return Ok(ArrowDataType::Decimal64(
+                                    precision_raw.parse::<usize>().map_err(|_| {
+                                        polars_err!(ComputeError: "Decimal precision is not a valid integer")
+                                    })?,
+                                    scale_raw.parse::<usize>().map_err(|_| {
+                                        polars_err!(ComputeError: "Decimal scale is not a valid integer")
+                                    })?,
+                                )),
+                                256 => return Ok(ArrowDataType::Decimal256(
+                                    precision_raw.parse::<usize>().map_err(|_| {
+                                        polars_err!(ComputeError: "Decimal precision is not a valid integer")
+                                    })?,
+                                    scale_raw.parse::<usize>().map_err(|_| {
+                                        polars_err!(ComputeError: "Decimal scale is not a valid integer")
+                                    })?,
+                                )),
+                                _ => {},
                             }
                             (precision_raw, scale_raw)
                         },
@@ -489,6 +506,8 @@ fn to_format(dtype: &ArrowDataType) -> String {
         ArrowDataType::Utf8View => "vu".to_string(),
         ArrowDataType::BinaryView => "vz".to_string(),
         ArrowDataType::Decimal(precision, scale) => format!("d:{precision},{scale}"),
+        ArrowDataType::Decimal32(precision, scale) => format!("d:{precision},{scale},32"),
+        ArrowDataType::Decimal64(precision, scale) => format!("d:{precision},{scale},64"),
         ArrowDataType::Decimal256(precision, scale) => format!("d:{precision},{scale},256"),
         ArrowDataType::List(_) => "+l".to_string(),
         ArrowDataType::LargeList(_) => "+L".to_string(),
