@@ -28,6 +28,17 @@ impl CategoricalChunked {
         self.physical_mut().null_count += other.null_count();
     }
 
+    pub fn take(&mut self) -> Self {
+        Self {
+            physical: Logical {
+                phys: core::mem::take(&mut self.physical.phys),
+                dtype: self.physical.dtype.clone(),
+                _phantom: PhantomData,
+            },
+            bit_settings: self.bit_settings.clone(),
+        }
+    }
+
     pub fn append(&mut self, other: &Self) -> PolarsResult<()> {
         polars_ensure!(!self.is_enum() || self.dtype() == other.dtype(), append);
 
