@@ -6,15 +6,13 @@ pub type DatetimeChunked = Logical<DatetimeType, Int64Type>;
 
 impl Int64Chunked {
     pub fn into_datetime(self, timeunit: TimeUnit, tz: Option<TimeZone>) -> DatetimeChunked {
-        let mut dt = DatetimeChunked::new_logical(self);
-        dt.dtype = Some(DataType::Datetime(timeunit, tz));
-        dt
+        DatetimeChunked::new_logical(self, DataType::Datetime(timeunit, tz))
     }
 }
 
 impl LogicalType for DatetimeChunked {
     fn dtype(&self) -> &DataType {
-        self.dtype.as_ref().unwrap()
+        &self.dtype
     }
 
     fn get_any_value(&self, i: usize) -> PolarsResult<AnyValue<'_>> {
@@ -54,7 +52,7 @@ impl LogicalType for DatetimeChunked {
                 };
                 match multiplier {
                     // scale to higher precision (eg: ms → us, ms → ns, us → ns)
-                    Some(m) => Ok((self.phys.as_ref() * m)
+                    Some(m) => Ok((self.phys.as_ref().checked_mul_scalar(m))
                         .into_datetime(*to_unit, tz.clone())
                         .into_series()),
                     // scale to lower precision (eg: ns → us, ns → ms, us → ms)
@@ -96,9 +94,9 @@ impl LogicalType for DatetimeChunked {
                 };
                 return Ok(self
                     .phys
-                    .apply_values(|v| {
-                        let t = v % scaled_mod * multiplier;
-                        t + (NS_IN_DAY * (t < 0) as i64)
+                    .apply(|v| {
+                        let t = (v? % scaled_mod).checked_mul(multiplier)?;
+                        t.checked_add(NS_IN_DAY * (t < 0) as i64)
                     })
                     .into_time()
                     .into_series());
