@@ -174,22 +174,19 @@ unsafe fn add_value<T: NumericNative>(
     row_idx: usize,
     value: T,
 ) {
-    let column = (*(values_buf_ptr as *mut Vec<Vec<T>>)).get_unchecked_mut(col_idx);
+    let vec_ref: &mut Vec<Vec<T>> = &mut *(values_buf_ptr as *mut Vec<Vec<T>>);
+    let column = vec_ref.get_unchecked_mut(col_idx);
     let el_ptr = column.as_mut_ptr();
     *el_ptr.add(row_idx) = value;
 }
 
 // This just fills a pre-allocated mutable series vector, which may have a name column.
 // Nothing is returned and the actual DataFrame is constructed above.
-pub(super) fn numeric_transpose<T>(
+pub(super) fn numeric_transpose<T: PolarsNumericType>(
     cols: &[Column],
     names_out: &[PlSmallStr],
     cols_t: &mut Vec<Column>,
-) where
-    T: PolarsNumericType,
-    //S: AsRef<str>,
-    ChunkedArray<T>: IntoSeries,
-{
+) {
     let new_width = cols[0].len();
     let new_height = cols.len();
 
@@ -225,8 +222,9 @@ pub(super) fn numeric_transpose<T>(
                     for (col_idx, opt_v) in ca.iter().enumerate() {
                         match opt_v {
                             None => unsafe {
-                                let column = (*(validity_buf_ptr as *mut Vec<Vec<bool>>))
-                                    .get_unchecked_mut(col_idx);
+                                let validity_vec: &mut Vec<Vec<bool>> =
+                                    &mut *(validity_buf_ptr as *mut Vec<Vec<bool>>);
+                                let column = validity_vec.get_unchecked_mut(col_idx);
                                 let el_ptr = column.as_mut_ptr();
                                 *el_ptr.add(row_idx) = false;
                                 // we must initialize this memory otherwise downstream code
@@ -242,9 +240,9 @@ pub(super) fn numeric_transpose<T>(
                 } else {
                     for (col_idx, v) in ca.into_no_null_iter().enumerate() {
                         unsafe {
-                            let column = (*(values_buf_ptr as *mut Vec<Vec<T::Native>>))
-                                .get_unchecked_mut(col_idx);
-                            let el_ptr = column.as_mut_ptr();
+                            let column: &mut Vec<Vec<T::Native>> =
+                                &mut *(values_buf_ptr as *mut Vec<Vec<T::Native>>);
+                            let el_ptr = column.get_unchecked_mut(col_idx).as_mut_ptr();
                             *el_ptr.add(row_idx) = v;
                         }
                     }
@@ -279,7 +277,7 @@ pub(super) fn numeric_transpose<T>(
                 values.into(),
                 validity,
             );
-            ChunkedArray::with_chunk(name.clone(), arr).into_column()
+            ChunkedArray::<T>::with_chunk(name.clone(), arr).into_column()
         });
     POOL.install(|| cols_t.par_extend(par_iter));
 }
