@@ -166,7 +166,7 @@ impl PhysicalExpr for AggregationExpr {
         let keep_name = ac.get_values().name().clone();
 
         // Literals cannot be aggregated except for implode.
-        polars_ensure!((!matches!(ac.agg_state(), AggState::Literal(_)) || matches!(self.agg_type.groupby, GroupByMethod::Implode)), ComputeError: "cannot aggregate a literal");
+        polars_ensure!(!matches!(ac.agg_state(), AggState::Literal(_)), ComputeError: "cannot aggregate a literal");
 
         if let AggregatedScalar(_) = ac.agg_state() {
             match self.agg_type.groupby {
