@@ -7,6 +7,7 @@ use bitflags::bitflags;
 use polars_core::config::verbose;
 use polars_core::prelude::*;
 use polars_ops::prelude::ChunkJoinOptIds;
+use polars_utils::unique_id::UniqueId;
 
 use super::NodeTimer;
 
@@ -68,9 +69,6 @@ bitflags! {
         const CACHE_WINDOW_EXPR = 0x02;
         /// Indicates the expression has a window function
         const HAS_WINDOW = 0x04;
-        /// If set, the expression is evaluated in the
-        /// streaming engine.
-        const IN_STREAMING = 0x08;
     }
 }
 
@@ -105,7 +103,7 @@ type CachedValue = Arc<(AtomicI64, OnceLock<DataFrame>)>;
 /// State/ cache that is maintained during the Execution of the physical plan.
 pub struct ExecutionState {
     // cached by a `.cache` call and kept in memory for the duration of the plan.
-    df_cache: Arc<RwLock<PlHashMap<usize, CachedValue>>>,
+    df_cache: Arc<RwLock<PlHashMap<UniqueId, CachedValue>>>,
     pub schema_cache: RwLock<Option<SchemaRef>>,
     /// Used by Window Expressions to cache intermediate state
     pub window_cache: Arc<WindowCache>,
@@ -216,17 +214,17 @@ impl ExecutionState {
         lock.clone()
     }
 
-    pub fn get_df_cache(&self, key: usize, cache_hits: u32) -> CachedValue {
+    pub fn get_df_cache(&self, key: &UniqueId, cache_hits: u32) -> CachedValue {
         let guard = self.df_cache.read().unwrap();
 
-        match guard.get(&key) {
+        match guard.get(key) {
             Some(v) => v.clone(),
             None => {
                 drop(guard);
                 let mut guard = self.df_cache.write().unwrap();
 
                 guard
-                    .entry(key)
+                    .entry(key.clone())
                     .or_insert_with(|| {
                         Arc::new((AtomicI64::new(cache_hits as i64), OnceLock::new()))
                     })
@@ -235,9 +233,9 @@ impl ExecutionState {
         }
     }
 
-    pub fn remove_df_cache(&self, key: usize) {
+    pub fn remove_df_cache(&self, key: &UniqueId) {
         let mut guard = self.df_cache.write().unwrap();
-        let _ = guard.remove(&key).unwrap();
+        let _ = guard.remove(key).unwrap();
     }
 
     /// Clear the cache used by the Window expressions
