@@ -2,10 +2,12 @@ use std::ops::Range;
 
 use bytes::Bytes;
 use futures::{StreamExt, TryStreamExt};
+use hashbrown::hash_map::RawEntryMut;
 use object_store::path::Path;
 use object_store::{ObjectMeta, ObjectStore};
 use polars_core::prelude::{InitHashMaps, PlHashMap};
 use polars_error::{PolarsError, PolarsResult};
+use polars_utils::mmap::MemSlice;
 use tokio::io::{AsyncSeekExt, AsyncWriteExt};
 
 use crate::pl_async::{
@@ -88,7 +90,7 @@ mod inner {
             // If this does not eq, then `inner` was already re-built by another thread.
             if Arc::ptr_eq(&*current_store, from_version) {
                 *current_store = self.inner.builder.clone().build_impl().await.map_err(|e| {
-                    e.wrap_msg(|e| format!("attempt to rebuild object store failed: {}", e))
+                    e.wrap_msg(|e| format!("attempt to rebuild object store failed: {e}"))
                 })?;
             }
 
@@ -119,7 +121,7 @@ mod inner {
             let store = self
                 .rebuild_inner(&store)
                 .await
-                .map_err(|e| e.wrap_msg(|e| format!("{}; original error: {}", e, orig_err)))?;
+                .map_err(|e| e.wrap_msg(|e| format!("{e}; original error: {orig_err}")))?;
 
             func(&store).await.map_err(|e| {
                 if self.inner.builder.is_azure()
@@ -130,10 +132,9 @@ mod inner {
                     // these keys exist only on the Python side.
                     e.wrap_msg(|e| {
                         format!(
-                            "{}; note: if you are using Python, consider setting \
+                            "{e}; note: if you are using Python, consider setting \
 POLARS_AUTO_USE_AZURE_STORAGE_ACCOUNT_KEY=1 if you would like polars to try to retrieve \
-and use the storage account keys from Azure CLI to authenticate",
-                            e
+and use the storage account keys from Azure CLI to authenticate"
                         )
                     })
                 } else {
@@ -215,14 +216,11 @@ impl PolarsObjectStore {
     ///
     /// # Panics
     /// Panics if the same range start is used by more than 1 range.
-    pub async fn get_ranges_sort<
-        K: TryFrom<usize, Error = impl std::fmt::Debug> + std::hash::Hash + Eq,
-        T: From<Bytes>,
-    >(
+    pub async fn get_ranges_sort(
         &self,
         path: &Path,
         ranges: &mut [Range<usize>],
-    ) -> PolarsResult<PlHashMap<K, T>> {
+    ) -> PolarsResult<PlHashMap<usize, MemSlice>> {
         if ranges.is_empty() {
             return Ok(Default::default());
         }
@@ -281,16 +279,23 @@ impl PolarsObjectStore {
 
                             assert_eq!(bytes.len(), full_range.len());
 
+                            let bytes = MemSlice::from_bytes(bytes);
+
                             for range in &ranges[current_offset..end] {
-                                let v = out.insert(
-                                    K::try_from(range.start).unwrap(),
-                                    T::from(bytes.slice(
-                                        range.start - full_range.start
-                                            ..range.end - full_range.start,
-                                    )),
+                                let mem_slice = bytes.slice(
+                                    range.start - full_range.start..range.end - full_range.start,
                                 );
 
-                                assert!(v.is_none()); // duplicate range start
+                                match out.raw_entry_mut().from_key(&range.start) {
+                                    RawEntryMut::Vacant(slot) => {
+                                        slot.insert(range.start, mem_slice);
+                                    },
+                                    RawEntryMut::Occupied(mut slot) => {
+                                        if slot.get_mut().len() < mem_slice.len() {
+                                            *slot.get_mut() = mem_slice;
+                                        }
+                                    },
+                                }
                             }
 
                             current_offset = end;
