@@ -47,7 +47,7 @@ pub static POLARS_TEMP_DIR_BASE_PATH: LazyLock<Box<Path>> = LazyLock::new(|| {
                 });
 
             if let Ok(v) = id {
-                std::env::temp_dir().join(format!("polars-{}/", v))
+                std::env::temp_dir().join(format!("polars-{v}/"))
             } else {
                 return Err(std::io::Error::other(
                     "could not load $USER or $HOME environment variables",
@@ -83,8 +83,7 @@ pub static POLARS_TEMP_DIR_BASE_PATH: LazyLock<Box<Path>> = LazyLock::new(|| {
 
                 if (perms.mode() % 0o1000) != 0o700 {
                     std::io::Result::Err(std::io::Error::other(format!(
-                        "permission mismatch: {:?}",
-                        perms
+                        "permission mismatch: {perms:?}"
                     )))
                 } else {
                     std::io::Result::Ok(())
@@ -112,9 +111,8 @@ pub static POLARS_TEMP_DIR_BASE_PATH: LazyLock<Box<Path>> = LazyLock::new(|| {
         std::io::Error::new(
             e.kind(),
             format!(
-                "error initializing temporary directory: {} \
-                 consider explicitly setting POLARS_TEMP_DIR",
-                e
+                "error initializing temporary directory: {e} \
+                 consider explicitly setting POLARS_TEMP_DIR"
             ),
         )
     })
@@ -313,9 +311,9 @@ pub fn expand_paths_hive(
 
             let format_path = |scheme: &str, bucket: &str, location: &str| {
                 if is_cloud {
-                    format!("{}://{}/{}", scheme, bucket, location)
+                    format!("{scheme}://{bucket}/{location}")
                 } else {
-                    format!("/{}", location)
+                    format!("/{location}")
                 }
             };
 
