@@ -598,7 +598,8 @@ fn test_anonymous_function_returns_scalar_all_null_20679() {
         input: vec![col("b")],
         function: LazySerde::Deserialized(SpecialEq::new(Arc::new(f))),
         output_type: Default::default(),
-        options: FunctionOptions::aggregation().with_fmt_str(""),
+        options: FunctionOptions::aggregation(),
+        fmt_str: Box::new(PlSmallStr::EMPTY),
     };
 
     let grouped_df = df
