@@ -1,6 +1,7 @@
 use std::hash::Hash;
 use std::sync::Mutex;
 
+use deletion::DeletionFilesList;
 use polars_core::utils::get_numeric_upcast_supertype_lossless;
 use polars_io::cloud::CloudOptions;
 #[cfg(feature = "csv")]
@@ -18,6 +19,7 @@ use serde::{Deserialize, Serialize};
 use strum_macros::IntoStaticStr;
 
 use super::*;
+pub mod deletion;
 
 #[cfg(feature = "python")]
 pub mod python_dataset;
@@ -33,6 +35,7 @@ bitflags::bitflags! {
 
 #[derive(Clone, Debug, IntoStaticStr)]
 #[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
+#[cfg_attr(feature = "dsl-schema", derive(schemars::JsonSchema))]
 // TODO: Arc<> some of the options and the cloud options.
 pub enum FileScan {
     #[cfg(feature = "csv")]
@@ -44,14 +47,14 @@ pub enum FileScan {
     #[cfg(feature = "parquet")]
     Parquet {
         options: ParquetOptions,
-        #[cfg_attr(feature = "serde", serde(skip))]
+        #[cfg_attr(any(feature = "serde", feature = "dsl-schema"), serde(skip))]
         metadata: Option<FileMetadataRef>,
     },
 
     #[cfg(feature = "ipc")]
     Ipc {
         options: IpcScanOptions,
-        #[cfg_attr(feature = "serde", serde(skip))]
+        #[cfg_attr(any(feature = "serde", feature = "dsl-schema"), serde(skip))]
         metadata: Option<Arc<arrow::io::ipc::read::FileMetadata>>,
     },
 
@@ -59,11 +62,11 @@ pub enum FileScan {
     PythonDataset {
         dataset_object: Arc<python_dataset::PythonDatasetProvider>,
 
-        #[cfg_attr(feature = "serde", serde(skip, default))]
+        #[cfg_attr(any(feature = "serde", feature = "dsl-schema"), serde(skip, default))]
         cached_ir: Arc<Mutex<Option<ExpandedDataset>>>,
     },
 
-    #[cfg_attr(feature = "serde", serde(skip))]
+    #[cfg_attr(any(feature = "serde", feature = "dsl-schema"), serde(skip))]
     Anonymous {
         options: Arc<AnonymousScanOptions>,
         function: Arc<dyn AnonymousScan>,
@@ -117,6 +120,7 @@ impl FileScan {
 
 #[derive(Debug, Clone, Copy, PartialEq, Eq, Default, Hash)]
 #[cfg_attr(feature = "serde", derive(serde::Serialize, serde::Deserialize))]
+#[cfg_attr(feature = "dsl-schema", derive(schemars::JsonSchema))]
 pub enum MissingColumnsPolicy {
     #[default]
     Raise,
@@ -127,6 +131,7 @@ pub enum MissingColumnsPolicy {
 /// Used by scans.
 #[derive(Debug, Clone, PartialEq, Eq, Hash)]
 #[cfg_attr(feature = "serde", derive(serde::Serialize, serde::Deserialize))]
+#[cfg_attr(feature = "dsl-schema", derive(schemars::JsonSchema))]
 pub struct CastColumnsPolicy {
     /// Allow casting when target dtype is lossless supertype
     pub integer_upcast: bool,
@@ -171,6 +176,7 @@ impl Default for CastColumnsPolicy {
 
 #[derive(Debug, Clone, Copy, PartialEq, Eq, Default, Hash)]
 #[cfg_attr(feature = "serde", derive(serde::Serialize, serde::Deserialize))]
+#[cfg_attr(feature = "dsl-schema", derive(schemars::JsonSchema))]
 pub enum ExtraColumnsPolicy {
     /// Error if there are extra columns outside the target schema.
     #[default]
@@ -179,8 +185,9 @@ pub enum ExtraColumnsPolicy {
 }
 
 /// Scan arguments shared across different scan types.
-#[derive(Debug, Clone, PartialEq, Eq, Default, Hash)]
+#[derive(Debug, Clone, PartialEq, Eq, Hash)]
 #[cfg_attr(feature = "serde", derive(serde::Serialize, serde::Deserialize))]
+#[cfg_attr(feature = "dsl-schema", derive(schemars::JsonSchema))]
 pub struct UnifiedScanArgs {
     /// User-provided schema of the file. Will be inferred during IR conversion
     /// if None.
@@ -199,7 +206,31 @@ pub struct UnifiedScanArgs {
 
     pub cast_columns_policy: CastColumnsPolicy,
     pub missing_columns_policy: MissingColumnsPolicy,
+    pub extra_columns_policy: ExtraColumnsPolicy,
     pub include_file_paths: Option<PlSmallStr>,
+
+    pub deletion_files: Option<DeletionFilesList>,
+}
+
+impl Default for UnifiedScanArgs {
+    fn default() -> Self {
+        Self {
+            schema: None,
+            cloud_options: None,
+            hive_options: HiveOptions::new_enabled(),
+            rechunk: false,
+            cache: false,
+            glob: true,
+            projection: None,
+            row_index: None,
+            pre_slice: None,
+            cast_columns_policy: CastColumnsPolicy::default(),
+            missing_columns_policy: MissingColumnsPolicy::default(),
+            extra_columns_policy: ExtraColumnsPolicy::default(),
+            include_file_paths: None,
+            deletion_files: None,
+        }
+    }
 }
 
 /// Manual impls of Eq/Hash, as some fields are `Arc<T>` where T does not have Eq/Hash. For these
@@ -410,7 +441,8 @@ impl CastColumnsPolicy {
                         ExtraColumnsPolicy::Raise => {
                             return mismatch_err(&format!(
                                 "encountered extra struct field: {}, \
-                                hint: pass cast_options=pl.ScanCastOptions(extra_struct_fields='ignore')",
+                                hint: specify this field in the schema, or pass \
+                                cast_options=pl.ScanCastOptions(extra_struct_fields='ignore')",
                                 &fld.name,
                             ));
                         },
