@@ -14,6 +14,7 @@ use crate::prelude::*;
 
 #[derive(Clone, PartialEq)]
 #[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
+#[cfg_attr(feature = "dsl-schema", derive(schemars::JsonSchema))]
 pub enum DynLiteralValue {
     Str(PlSmallStr),
     Int(i128),
@@ -22,6 +23,7 @@ pub enum DynLiteralValue {
 }
 #[derive(Clone, PartialEq)]
 #[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
+#[cfg_attr(feature = "dsl-schema", derive(schemars::JsonSchema))]
 pub enum DynListLiteralValue {
     Str(Box<[Option<PlSmallStr>]>),
     Int(Box<[Option<i128>]>),
@@ -57,6 +59,7 @@ impl Hash for DynListLiteralValue {
 
 #[derive(Clone, PartialEq, Hash)]
 #[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
+#[cfg_attr(feature = "dsl-schema", derive(schemars::JsonSchema))]
 pub struct RangeLiteralValue {
     pub low: i128,
     pub high: i128,
@@ -64,6 +67,7 @@ pub struct RangeLiteralValue {
 }
 #[derive(Clone, PartialEq)]
 #[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
+#[cfg_attr(feature = "dsl-schema", derive(schemars::JsonSchema))]
 pub enum LiteralValue {
     /// A dynamically inferred literal value. This needs to be materialized into a specific type.
     Dyn(DynLiteralValue),
@@ -134,6 +138,9 @@ impl DynLiteralValue {
                 Ok(Scalar::from(s).cast_with_options(dtype, CastOptions::Strict)?)
             },
             DynLiteralValue::Int(i) => {
+                #[cfg(not(feature = "dtype-i128"))]
+                let i: i64 = i.try_into().expect("activate dtype-i128 feature");
+
                 Ok(Scalar::from(i).cast_with_options(dtype, CastOptions::Strict)?)
             },
             DynLiteralValue::Float(f) => {
@@ -201,14 +208,6 @@ impl RangeLiteralValue {
 }
 
 impl LiteralValue {
-    /// Get the output name as `&str`.
-    pub(crate) fn output_name(&self) -> &PlSmallStr {
-        match self {
-            LiteralValue::Series(s) => s.name(),
-            _ => get_literal_name(),
-        }
-    }
-
     /// Get the output name as [`PlSmallStr`].
     pub(crate) fn output_column_name(&self) -> &PlSmallStr {
         match self {
@@ -380,6 +379,24 @@ impl LiteralValue {
     pub const fn untyped_null() -> Self {
         Self::Scalar(Scalar::null(DataType::Null))
     }
+
+    pub fn implode(self) -> PolarsResult<Self> {
+        let series = match self.materialize() {
+            LiteralValue::Dyn(_) => unreachable!(),
+            LiteralValue::Scalar(scalar) => scalar.into_series(PlSmallStr::EMPTY),
+            LiteralValue::Series(series) => series.into_inner(),
+            LiteralValue::Range(range) => {
+                let dtype = range.dtype.clone();
+                range.try_materialize_to_series(&dtype)?
+            },
+        };
+
+        let dtype = DataType::List(Box::new(series.dtype().clone()));
+        Ok(LiteralValue::Scalar(Scalar::new(
+            dtype,
+            AnyValue::List(series),
+        )))
+    }
 }
 
 impl From<Scalar> for LiteralValue {
@@ -553,8 +570,7 @@ impl Literal for Duration {
     fn lit(self) -> Expr {
         assert!(
             self.months() == 0,
-            "Cannot create literal duration that is not of fixed length; found {}",
-            self
+            "Cannot create literal duration that is not of fixed length; found {self}"
         );
         let ns = self.duration_ns();
         Expr::Literal(
