@@ -1,7 +1,6 @@
 mod group_by;
 mod join;
 mod keys;
-mod rename;
 mod utils;
 
 use polars_core::datatypes::PlHashMap;
@@ -9,13 +8,9 @@ use polars_core::prelude::*;
 use recursive::recursive;
 use utils::*;
 
-#[cfg(feature = "python")]
-use self::python_dsl::PythonScanSource;
 use super::*;
-use crate::dsl::function_expr::FunctionExpr;
 use crate::prelude::optimizer::predicate_pushdown::group_by::process_group_by;
 use crate::prelude::optimizer::predicate_pushdown::join::process_join;
-use crate::prelude::optimizer::predicate_pushdown::rename::process_rename;
 use crate::utils::{check_input_node, has_aexpr};
 
 pub type ExprEval<'a> =
@@ -495,17 +490,6 @@ impl PredicatePushDown<'_> {
             MapFunction { ref function, .. } => {
                 if function.allow_predicate_pd() {
                     match function {
-                        FunctionIR::Rename { existing, new, .. } => {
-                            process_rename(&mut acc_predicates, expr_arena, existing, new);
-
-                            self.pushdown_and_continue(
-                                lp,
-                                acc_predicates,
-                                lp_arena,
-                                expr_arena,
-                                false,
-                            )
-                        },
                         FunctionIR::Explode { columns, .. } => {
                             let condition = |name: &PlSmallStr| columns.iter().any(|s| s == name);
 
@@ -658,22 +642,31 @@ impl PredicatePushDown<'_> {
             PythonScan { mut options } => {
                 let predicate = predicate_at_scan(acc_predicates, None, expr_arena);
                 if let Some(predicate) = predicate {
-                    // For IO plugins we only accept streamable expressions as
-                    // we want to apply the predicates to the batches.
-                    if !is_elementwise_rec(predicate.node(), expr_arena)
-                        && matches!(options.python_source, PythonScanSource::IOPlugin)
-                    {
-                        let lp = PythonScan { options };
-                        return Ok(self.optional_apply_predicate(
-                            lp,
-                            vec![predicate],
-                            lp_arena,
-                            expr_arena,
-                        ));
-                    }
+                    match ExprPushdownGroup::Pushable.update_with_expr_rec(
+                        expr_arena.get(predicate.node()),
+                        expr_arena,
+                        None,
+                    ) {
+                        ExprPushdownGroup::Barrier => {
+                            if cfg!(debug_assertions) {
+                                // Expression should not be pushed here by the optimizer
+                                panic!()
+                            }
 
-                    options.predicate = PythonPredicate::Polars(predicate);
+                            return Ok(self.optional_apply_predicate(
+                                PythonScan { options },
+                                vec![predicate],
+                                lp_arena,
+                                expr_arena,
+                            ));
+                        },
+
+                        ExprPushdownGroup::Pushable | ExprPushdownGroup::Fallible => {
+                            options.predicate = PythonPredicate::Polars(predicate);
+                        },
+                    }
                 }
+
                 Ok(PythonScan { options })
             },
             #[cfg(feature = "merge_sorted")]
