@@ -144,8 +144,7 @@ impl FileReader for NDJsonFileReader {
 
                     if verbose {
                         eprintln!(
-                            "[NDJsonFileReader]: n_lines_estimate: {}, line_length_estimate: {}",
-                            n_lines_estimate, line_length_estimate
+                            "[NDJsonFileReader]: n_lines_estimate: {n_lines_estimate}, line_length_estimate: {line_length_estimate}"
                         );
                     }
 
@@ -400,8 +399,7 @@ impl FileReader for NDJsonFileReader {
                 if verbose {
                     eprintln!(
                         "[NDJsonFileReader]: \
-                        send total row count: {}",
-                        total_row_count
+                        send total row count: {total_row_count}"
                     )
                 }
                 _ = tx.send(total_row_count);
@@ -411,10 +409,7 @@ impl FileReader for NDJsonFileReader {
                 let total_row_count = total_row_count.unwrap();
 
                 if verbose {
-                    eprintln!(
-                        "[NDJsonFileReader]: send n_rows_in_file: {}",
-                        total_row_count
-                    );
+                    eprintln!("[NDJsonFileReader]: send n_rows_in_file: {total_row_count}");
                 }
 
                 let num_rows = total_row_count;
