@@ -76,6 +76,21 @@ impl PartialEq for PythonObject {
     }
 }
 
+#[cfg(feature = "dsl-schema")]
+impl schemars::JsonSchema for PythonObject {
+    fn schema_name() -> String {
+        "PythonObject".to_owned()
+    }
+
+    fn schema_id() -> std::borrow::Cow<'static, str> {
+        std::borrow::Cow::Borrowed(concat!(module_path!(), "::", "PythonObject"))
+    }
+
+    fn json_schema(generator: &mut schemars::r#gen::SchemaGenerator) -> schemars::schema::Schema {
+        Vec::<u8>::json_schema(generator)
+    }
+}
+
 #[cfg(feature = "serde")]
 mod _serde_impls {
     use super::{PySerializeWrap, PythonObject, TrySerializeToBytes, serde_wrap};
@@ -254,8 +269,7 @@ mod serde_wrap {
                     if config::verbose() {
                         eprintln!(
                             "serialize_pyobject_with_cloudpickle_fallback(): \
-                            retrying with cloudpickle due to error: {:?}",
-                            e
+                            retrying with cloudpickle due to error: {e:?}"
                         );
                     }
 
